/*-
 * %%----------------------------------------------------------------------------------------------
 * Archive ouverte UNIGE - AoU Resource Server Common - AbstractPermissionService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2022 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.aou.service.security;

import java.util.Collection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import ch.unige.solidify.auth.model.ApplicationRole;
import ch.unige.solidify.auth.model.AuthApplicationRole;
import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.controller.AouControllerAction;

public abstract class AbstractPermissionService {

  private static final Logger logger = LoggerFactory.getLogger(AbstractPermissionService.class);
  
  protected boolean canBypassPermissionCheck() {
    final String[] authorizedApplicationRoleNames = { AuthApplicationRole.ADMIN_ID, AuthApplicationRole.ROOT_ID,
            AuthApplicationRole.TRUSTED_CLIENT_ID };

    final Collection<? extends GrantedAuthority> authorities = this.getAuthentication().getAuthorities();

    for (final String roleName : authorizedApplicationRoleNames) {
      for (final GrantedAuthority authority : authorities) {
        if (authority.toString().equals(roleName)) {
          return true;
        }
      }
    }
    return false;
  }

  protected boolean hasUserApplicationRole() {
    return this.hasApplicationRole(AuthApplicationRole.USER);
  }

  protected boolean hasApplicationRole(ApplicationRole applicationRole) {
    final Collection<? extends GrantedAuthority> authorities = this.getAuthentication().getAuthorities();
    return authorities.stream().anyMatch(a -> a.toString().equals(applicationRole.getResId()));
  }

  protected Authentication getAuthentication() {
    return SecurityContextHolder.getContext().getAuthentication();
  }

  protected AouControllerAction getControllerAction(Object permission) {
    try {
      return AouControllerAction.valueOf(permission.toString());
    } catch (final Exception ex) {
      logger.error("Unable to check for access permission: the action name '{}' is wrong", permission);
      throw new SolidifyRuntimeException("unable to check for access permission: the action name '" + permission + "' is wrong");
    }
  }
}
